@@ -3,9 +3,14 @@ from typing import (Callable,
                     TypeVar)
 
 from ground.base import Orientation
-from ground.hints import Point
+from ground.hints import (Contour,
+                          Point,
+                          Scalar)
+from hypothesis.strategies import SearchStrategy as Strategy
 
 Domain = TypeVar('Domain')
 Range = TypeVar('Range')
 Chooser = Callable[[Sequence[Domain]], Domain]
+Multicontour = Sequence[Contour[Scalar]]
 Orienteer = Callable[[Point, Point, Point], Orientation]
+Strategy = Strategy
