@@ -37,6 +37,9 @@ from .core.base import (boxes as _boxes,
                         as _triangular_vertices_sequences,
                         vertices_sequences as _vertices_sequences)
 from .core.constants import (MIN_CONTOUR_SIZE as _MIN_CONTOUR_SIZE,
+                             MIN_MIX_COMPONENTS_COUNT
+                             as _MIN_MIX_COMPONENTS_COUNT,
+                             MIN_MULTICONTOUR_SIZE as _MIN_MULTICONTOUR_SIZE,
                              MIN_MULTIPOINT_SIZE as _MIN_MULTIPOINT_SIZE,
                              MIN_MULTIPOLYGON_SIZE as _MIN_MULTIPOLYGON_SIZE,
                              MIN_MULTISEGMENT_SIZE as _MIN_MULTISEGMENT_SIZE,
@@ -66,7 +69,7 @@ def empty_geometries(context: _Optional[_Context] = None) -> _Strategy[_Empty]:
 def points(x_coordinates: _Strategy[_Scalar],
            y_coordinates: _Optional[_Strategy[_Scalar]] = None,
            *,
-           context: _Optional[_Context] = None) -> _Strategy[_Point]:
+           context: _Optional[_Context] = None) -> _Strategy[_Point[_Scalar]]:
     """
     Returns a strategy for points.
 
@@ -131,7 +134,8 @@ def multipoints(x_coordinates: _Strategy[_Scalar],
                 *,
                 min_size: int = _MIN_MULTIPOINT_SIZE,
                 max_size: _Optional[int] = None,
-                context: _Optional[_Context] = None) -> _Strategy[_Multipoint]:
+                context: _Optional[_Context] = None
+                ) -> _Strategy[_Multipoint[_Scalar]]:
     """
     Returns a strategy for multipoints.
 
@@ -215,7 +219,8 @@ def multipoints(x_coordinates: _Strategy[_Scalar],
 def segments(x_coordinates: _Strategy[_Scalar],
              y_coordinates: _Optional[_Strategy[_Scalar]] = None,
              *,
-             context: _Optional[_Context] = None) -> _Strategy[_Segment]:
+             context: _Optional[_Context] = None
+             ) -> _Strategy[_Segment[_Scalar]]:
     """
     Returns a strategy for segments.
 
@@ -289,7 +294,7 @@ def multisegments(x_coordinates: _Strategy[_Scalar],
                   min_size: int = _MIN_MULTISEGMENT_SIZE,
                   max_size: _Optional[int] = None,
                   context: _Optional[_Context] = None
-                  ) -> _Strategy[_Multisegment]:
+                  ) -> _Strategy[_Multisegment[_Scalar]]:
     """
     Returns a strategy for multisegments.
 
@@ -383,7 +388,8 @@ def contours(x_coordinates: _Strategy[_Scalar],
              *,
              min_size: int = _MIN_CONTOUR_SIZE,
              max_size: _Optional[int] = None,
-             context: _Optional[_Context] = None) -> _Strategy[_Contour]:
+             context: _Optional[_Context] = None
+             ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for contours.
 
@@ -471,7 +477,7 @@ def convex_contours(x_coordinates: _Strategy[_Scalar],
                     min_size: int = _MinContourSize.CONVEX,
                     max_size: _Optional[int] = None,
                     context: _Optional[_Context] = None
-                    ) -> _Strategy[_Contour]:
+                    ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for convex contours.
     Convex contour is a contour such that the line segment
@@ -563,7 +569,7 @@ def concave_contours(x_coordinates: _Strategy[_Scalar],
                      min_size: int = _MinContourSize.CONCAVE,
                      max_size: _Optional[int] = None,
                      context: _Optional[_Context] = None
-                     ) -> _Strategy[_Contour]:
+                     ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for concave contours.
     Concave contour is a contour that is not convex.
@@ -651,7 +657,7 @@ def triangular_contours(x_coordinates: _Strategy[_Scalar],
                         y_coordinates: _Optional[_Strategy[_Scalar]]
                         = None,
                         context: _Optional[_Context] = None
-                        ) -> _Strategy[_Contour]:
+                        ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for triangular contours.
     Triangular contour is a contour formed by 3 points.
@@ -727,7 +733,7 @@ def rectangular_contours(x_coordinates: _Strategy[_Scalar],
                          y_coordinates: _Optional[_Strategy[_Scalar]]
                          = None,
                          context: _Optional[_Context] = None
-                         ) -> _Strategy[_Contour]:
+                         ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for axis-aligned rectangular contours.
     Rectangular contour is a contour formed by 4 points.
@@ -801,7 +807,7 @@ def rectangular_contours(x_coordinates: _Strategy[_Scalar],
 
 def boxes(x_coordinates: _Strategy[_Scalar],
           y_coordinates: _Optional[_Strategy[_Scalar]] = None,
-          context: _Optional[_Context] = None) -> _Strategy[_Box]:
+          context: _Optional[_Context] = None) -> _Strategy[_Box[_Scalar]]:
     """
     Returns a strategy for boxes.
 
@@ -875,7 +881,8 @@ def star_contours(x_coordinates: _Strategy[_Scalar],
                   *,
                   min_size: int = _MIN_CONTOUR_SIZE,
                   max_size: _Optional[int] = None,
-                  context: _Optional[_Context] = None) -> _Strategy[_Contour]:
+                  context: _Optional[_Context] = None
+                  ) -> _Strategy[_Contour[_Scalar]]:
     """
     Returns a strategy for star contours.
     Star contour is a contour such that every vertex is visible from centroid,
@@ -962,15 +969,15 @@ def star_contours(x_coordinates: _Strategy[_Scalar],
 def multicontours(x_coordinates: _Strategy[_Scalar],
                   y_coordinates: _Optional[_Strategy[_Scalar]] = None,
                   *,
-                  min_size: int = 0,
+                  min_size: int = _MIN_MULTICONTOUR_SIZE,
                   max_size: _Optional[int] = None,
                   min_contour_size: int = _MIN_CONTOUR_SIZE,
                   max_contour_size: _Optional[int] = None,
                   context: _Optional[_Context] = None
-                  ) -> _Strategy[_Multicontour]:
+                  ) -> _Strategy[_Multicontour[_Scalar]]:
     """
     Returns a strategy for multicontours.
-    Multicontour is a possibly empty sequence of non-crossing
+    Multicontour is a sequence of two or more non-crossing
     and non-overlapping contours.
 
     :param x_coordinates: strategy for vertices' x-coordinates.
@@ -1064,11 +1071,11 @@ def multicontours(x_coordinates: _Strategy[_Scalar],
     ...     for vertex in contour.vertices)
     True
     """
-    _validate_sizes(min_size, max_size, 0)
+    _validate_sizes(min_size, max_size, _MIN_MULTICONTOUR_SIZE)
     _validate_sizes(min_contour_size, max_contour_size, _MIN_CONTOUR_SIZE,
                     'min_contour_size', 'max_contour_size')
     return _multicontours(x_coordinates, y_coordinates,
-                          min_size=max(min_size, 0),
+                          min_size=max(min_size, _MIN_MULTICONTOUR_SIZE),
                           max_size=max_size,
                           min_contour_size=max(min_contour_size,
                                                _MIN_CONTOUR_SIZE),
@@ -1087,7 +1094,8 @@ def polygons(x_coordinates: _Strategy[_Scalar],
              max_holes_size: _Optional[int] = None,
              min_hole_size: int = _MIN_CONTOUR_SIZE,
              max_hole_size: _Optional[int] = None,
-             context: _Optional[_Context] = None) -> _Strategy[_Polygon]:
+             context: _Optional[_Context] = None
+             ) -> _Strategy[_Polygon[_Scalar]]:
     """
     Returns a strategy for polygons.
 
@@ -1232,7 +1240,7 @@ def multipolygons(x_coordinates: _Strategy[_Scalar],
                   min_hole_size: int = _MIN_CONTOUR_SIZE,
                   max_hole_size: _Optional[int] = None,
                   context: _Optional[_Context] = None
-                  ) -> _Strategy[_Multipolygon]:
+                  ) -> _Strategy[_Multipolygon[_Scalar]]:
     """
     Returns a strategy for multipolygons.
 
@@ -1400,10 +1408,9 @@ def mixes(x_coordinates: _Strategy[_Scalar],
           max_polygon_holes_size: _Optional[int] = None,
           min_polygon_hole_size: int = _MIN_CONTOUR_SIZE,
           max_polygon_hole_size: _Optional[int] = None,
-          context: _Optional[_Context] = None) -> _Strategy[_Mix]:
+          context: _Optional[_Context] = None) -> _Strategy[_Mix[_Scalar]]:
     """
     Returns a strategy for mixes.
-    Mix is a triplet of disjoint multipoint, multisegment and multipolygon.
 
     :param x_coordinates: strategy for vertices' x-coordinates.
     :param y_coordinates:
@@ -1492,7 +1499,7 @@ def mixes(x_coordinates: _Strategy[_Scalar],
     ...                   if isinstance(mix.linear, Segment)
     ...                   else (mix.linear.segments
     ...                         if isinstance(mix.linear, Multisegment)
-    ...                         else context.contour_edges(mix.linear))))
+    ...                         else context.contour_segments(mix.linear))))
     >>> min_segments_size <= len(segments) <= max_segments_size
     True
     >>> all(isinstance(segment.start.x, coordinates_type)
@@ -1601,7 +1608,7 @@ def mixes(x_coordinates: _Strategy[_Scalar],
     ...                   if isinstance(mix.linear, Segment)
     ...                   else (mix.linear.segments
     ...                         if isinstance(mix.linear, Multisegment)
-    ...                         else context.contour_edges(mix.linear))))
+    ...                         else context.contour_segments(mix.linear))))
     >>> min_segments_size <= len(segments) <= max_segments_size
     True
     >>> all(isinstance(segment.start.x, coordinates_type)
@@ -1670,25 +1677,111 @@ def mixes(x_coordinates: _Strategy[_Scalar],
     _validate_sizes(min_polygon_hole_size, max_polygon_hole_size,
                     _MIN_CONTOUR_SIZE, 'min_polygon_hole_size',
                     'max_polygon_hole_size')
-    return _mixes(x_coordinates, y_coordinates,
-                  min_points_size=max(min_points_size, 0),
-                  max_points_size=max_points_size,
-                  min_segments_size=max(min_segments_size, 0),
-                  max_segments_size=max_segments_size,
-                  min_polygons_size=max(min_polygons_size, 0),
-                  max_polygons_size=max_polygons_size,
-                  min_polygon_border_size=max(min_polygon_border_size,
-                                              _MIN_CONTOUR_SIZE),
-                  max_polygon_border_size=max_polygon_border_size,
-                  min_polygon_holes_size=max(min_polygon_holes_size, 0),
-                  max_polygon_holes_size=max_polygon_holes_size,
-                  min_polygon_hole_size=max(min_polygon_hole_size,
-                                            _MIN_CONTOUR_SIZE),
-                  max_polygon_hole_size=max_polygon_hole_size,
-                  context=_get_context() if context is None else context)
+    has_discrete = max_points_size is None or bool(max_points_size)
+    has_linear = max_segments_size is None or bool(max_segments_size)
+    has_shaped = max_polygons_size is None or bool(max_polygons_size)
+    if has_discrete + has_linear + has_shaped < _MIN_MIX_COMPONENTS_COUNT:
+        raise ValueError('Mix should have at least {min_count} components.'
+                         .format(min_count=_MIN_MIX_COMPONENTS_COUNT))
+    min_points_size = max(min_points_size, 0)
+    min_segments_size = max(min_segments_size, 0)
+    min_polygons_size = max(min_polygons_size, 0)
+    min_polygon_border_size = max(min_polygon_border_size, _MIN_CONTOUR_SIZE)
+    min_polygon_holes_size = max(min_polygon_holes_size, 0)
+    min_polygon_hole_size = max(min_polygon_hole_size, _MIN_CONTOUR_SIZE)
+    if context is None:
+        context = _get_context()
+    return (((_mixes(x_coordinates, y_coordinates,
+                     min_points_size=max(min_points_size, 1),
+                     max_points_size=max_points_size,
+                     min_segments_size=max(min_segments_size, 1),
+                     max_segments_size=max_segments_size,
+                     min_polygons_size=min_polygons_size,
+                     max_polygons_size=max_polygons_size,
+                     min_polygon_border_size=min_polygon_border_size,
+                     max_polygon_border_size=max_polygon_border_size,
+                     min_polygon_holes_size=min_polygon_holes_size,
+                     max_polygon_holes_size=max_polygon_holes_size,
+                     min_polygon_hole_size=min_polygon_hole_size,
+                     max_polygon_hole_size=max_polygon_hole_size,
+                     context=context)
+              | _mixes(x_coordinates, y_coordinates,
+                       min_points_size=max(min_points_size, 1),
+                       max_points_size=max_points_size,
+                       min_segments_size=min_segments_size,
+                       max_segments_size=max_segments_size,
+                       min_polygons_size=max(min_polygons_size, 1),
+                       max_polygons_size=max_polygons_size,
+                       min_polygon_border_size=min_polygon_border_size,
+                       max_polygon_border_size=max_polygon_border_size,
+                       min_polygon_holes_size=min_polygon_holes_size,
+                       max_polygon_holes_size=max_polygon_holes_size,
+                       min_polygon_hole_size=min_polygon_hole_size,
+                       max_polygon_hole_size=max_polygon_hole_size,
+                       context=context)
+              | _mixes(x_coordinates, y_coordinates,
+                       min_points_size=min_points_size,
+                       max_points_size=max_points_size,
+                       min_segments_size=max(min_segments_size, 1),
+                       max_segments_size=max_segments_size,
+                       min_polygons_size=max(min_polygons_size, 1),
+                       max_polygons_size=max_polygons_size,
+                       min_polygon_border_size=min_polygon_border_size,
+                       max_polygon_border_size=max_polygon_border_size,
+                       min_polygon_holes_size=min_polygon_holes_size,
+                       max_polygon_holes_size=max_polygon_holes_size,
+                       min_polygon_hole_size=min_polygon_hole_size,
+                       max_polygon_hole_size=max_polygon_hole_size,
+                       context=context)
+              if has_shaped
+              else _mixes(x_coordinates, y_coordinates,
+                          min_points_size=max(min_points_size, 1),
+                          max_points_size=max_points_size,
+                          min_segments_size=max(min_segments_size, 1),
+                          max_segments_size=max_segments_size,
+                          min_polygons_size=min_polygons_size,
+                          max_polygons_size=max_polygons_size,
+                          min_polygon_border_size=min_polygon_border_size,
+                          max_polygon_border_size=max_polygon_border_size,
+                          min_polygon_holes_size=min_polygon_holes_size,
+                          max_polygon_holes_size=max_polygon_holes_size,
+                          min_polygon_hole_size=min_polygon_hole_size,
+                          max_polygon_hole_size=max_polygon_hole_size,
+                          context=context))
+             if has_linear
+             else _mixes(x_coordinates, y_coordinates,
+                         min_points_size=max(min_points_size, 1),
+                         max_points_size=max_points_size,
+                         min_segments_size=min_segments_size,
+                         max_segments_size=max_segments_size,
+                         min_polygons_size=max(min_polygons_size, 1),
+                         max_polygons_size=max_polygons_size,
+                         min_polygon_border_size=min_polygon_border_size,
+                         max_polygon_border_size=max_polygon_border_size,
+                         min_polygon_holes_size=min_polygon_holes_size,
+                         max_polygon_holes_size=max_polygon_holes_size,
+                         min_polygon_hole_size=min_polygon_hole_size,
+                         max_polygon_hole_size=max_polygon_hole_size,
+                         context=context))
+            if has_discrete
+            else _mixes(x_coordinates, y_coordinates,
+                        min_points_size=min_points_size,
+                        max_points_size=max_points_size,
+                        min_segments_size=max(min_segments_size, 1),
+                        max_segments_size=max_segments_size,
+                        min_polygons_size=max(min_polygons_size, 1),
+                        max_polygons_size=max_polygons_size,
+                        min_polygon_border_size=min_polygon_border_size,
+                        max_polygon_border_size=max_polygon_border_size,
+                        min_polygon_holes_size=min_polygon_holes_size,
+                        max_polygon_holes_size=max_polygon_holes_size,
+                        min_polygon_hole_size=min_polygon_hole_size,
+                        max_polygon_hole_size=max_polygon_hole_size,
+                        context=context))
 
 
-def _validate_sizes(min_size: int, max_size: _Optional[int],
+def _validate_sizes(min_size: int,
+                    max_size: _Optional[int],
                     min_expected_size: int,
                     min_size_name: str = 'min_size',
                     max_size_name: str = 'max_size') -> None:
